function build_path(originPath, relativePath) {
    var full = null;
    if (relativePath.startsWith('/')) {
        full = relativePath;
    }
    else {
        var index = originPath.lastIndexOf('/');
        var dir = originPath.substr(0, index + 1);
        full = dir + relativePath;
    }
    var parts = full.split('/');
    var newParts = [];

    for (var i = 0; i < parts.length; i++) {
        var e = parts[i];
        switch (e) {
            case '..':
                newParts.pop();
                break;

            case '.':
                // do nothing
                break;

            default:
                newParts.push(e);
                break;
        }
    }
    return newParts.join('/');
}

function fileUrl_to_path(fileUrl) {
    var prefix = 'file://';
    if (fileUrl.startsWith(prefix)) {
        return fileUrl.substr(prefix.length, fileUrl.length);
    }
    return fileUrl;
}

var instanceModules = {};

service.register(options.serviceName, {
    create: function (id, env, config) {
        return {
            instance: {
                lccore: {
                    require: function (name, weex) {
                        if (!name.endsWith('.js')) {
                            name += '.js';
                        }
                        var thisUrl = weex.config.moduleUrl ? weex.config.moduleUrl : weex.config.bundleUrl;
                        var moduleUrl = build_path(thisUrl, name);

                        var modules = instanceModules[id];
                        if (!modules) {
                            modules = {};
                            instanceModules[id] = modules;
                        }
                        var nextModule = modules[moduleUrl];
                        if (!nextModule) {
                            var newWeex = {};
                            Object.assign(newWeex, weex);
                            newWeex.config = Object.assign({}, weex.config);
                            newWeex.config.moduleUrl = moduleUrl;

                            var globalKeys = [];
                            var globalValues = [];
                            for (var key in env.services) {
                                var s = env.services[key];
                                globalKeys.push(key);
                                globalValues.push(s);
                            }
                            globalKeys.push('weex');
                            globalValues.push(newWeex);

                            var fs = weex.requireModule('lc_fs');
                            var js = fs.readFileSync(fileUrl_to_path(moduleUrl));
                            globalKeys.push('return ' + js);

                            var func = new Function(...globalKeys);
                            nextModule = func(...globalValues);
                            modules[moduleUrl] = nextModule;
                        }
                        return nextModule;
                    }
                }
            }
        }
    },
    destroy: function (id, env) {
        if (instanceModules[id]) {
            delete instanceModules[id];
        }
    }
})
