package com.cndatacom.datas;

import java.util.ArrayList;
import java.util.List;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;

import com.cndatacom.utils.Strings;

public class DBManager
{
	private Context _Context = null;
	private DBHelper _DBHelper = null;
	private String _TABLE_AUTH = "com.cndatacom.utils.database";

	public DBManager(Context _Context, String dbProvider, String dbName, int dbVersion)
	{
		super();
		this._Context = _Context;
		this._TABLE_AUTH = dbProvider;
		this._DBHelper = new DBHelper(_Context, dbName, dbVersion);
	}
	
	public Uri getTableURI(String tableName)
	{
		return Uri.parse("content://" + _TABLE_AUTH + "/" + tableName);
	}

	public void TranAction(String sql) 
	{
		SQLiteDatabase db = _DBHelper.getWritableDatabase();

		db.beginTransaction();
		try 
		{
			db.execSQL(sql);
			db.setTransactionSuccessful();
		} 
		catch (Exception ex) 
		{
			ex.printStackTrace();
		} 
		finally 
		{
			db.endTransaction();
			db.close();
			_DBHelper.close();
		}
	}
	
	public List<KeysModel> getKeysList(int parentId)
	{
		StringBuffer sb = new StringBuffer();
		sb.append("Select [Id],[ParentId],[Index],[Key],[Value] From [Keys] Where [ParentId]=" + parentId);
		SQLiteDatabase db = _DBHelper.getWritableDatabase();
		Cursor c = db.rawQuery(sb.toString(), null);
		List<KeysModel> list = new ArrayList<KeysModel>();
		while (c.moveToNext()) 
		{
			KeysModel model = getKeysModel(c);
			list.add(model);
		}
		c.close();
		db.close();
		_DBHelper.close();
		return list;
	}

	public KeysModel getKeysById(String id)
	{		
		String sql = "Select [Id],[ParentId],[Index],[Key],[Value] From [Keys] Where [Id]=" + Strings.sqlClean(id) + " ";

		SQLiteDatabase db = _DBHelper.getWritableDatabase();
		Cursor c = db.rawQuery(sql, null);
		KeysModel model = null;
		if (c.moveToNext()) {			
			model = getKeysModel(c);
		}
		c.close();
		db.close();
		_DBHelper.close();
		
		return model;
	}
	
	public boolean insertKeys(KeysModel model)
	{		
		ContentResolver content = _Context.getContentResolver();
		ContentValues values = getKeysContents(model);		
		return (null != content.insert(getTableURI("Keys"), values));
	}
	
	public int updateKeys(KeysModel model, String id) 
	{		
		ContentResolver content = _Context.getContentResolver();
		String sFilters = " id = '" + Strings.sqlClean(id) + "' ";
		ContentValues values = getKeysContents(model);
		return content.update(getTableURI("Keys"), values, sFilters, null);
	}
	
	public int deleteKeys(String idList)
	{	
		idList = " [id] IN (" + Strings.sqlSList(idList) + ") ";
		return _Context.getContentResolver().delete(getTableURI("Keys"), idList, null);
	}
	
	private ContentValues getKeysContents(KeysModel model)
	{
		ContentValues values = new ContentValues();
		
		values.put("Id", model.getId());
		values.put("ParentId", model.getParentId());
		values.put("Index", model.getIndex());
		values.put("Key", model.getKey());
		values.put("Value", model.getValue());
		
		return values;
	}

	private KeysModel getKeysModel(Cursor c) 
	{
		KeysModel model = new KeysModel();
		
		model.setId(c.getInt(0));
		model.setParentId(c.getInt(1));
		model.setIndex(c.getInt(2));
		if (!c.isNull(3)){
			model.setKey(c.getString(3));
		}
		if (!c.isNull(4)){
			model.setValue(c.getString(4));
		}
		
		return model;
	}
	
	public class KeysModel
	{
		private int Id;
		private int ParentId;
		private int Index;
		private String Key;
		private String Value;
		
		public int getId(){return Id;}
		public void setId(int v){Id = v;}
		
		public int getParentId(){return ParentId;}
		public void setParentId(int v){ParentId = v;}

		public int getIndex(){return Index;}
		public void setIndex(int v){Index = v;}
		
		public String getKey(){return Key;}
		public void setKey(String v){Key = v;}
		
		public String getValue(){return Value;}
		public void setValue(String v){Value = v;}
	}
}
