package com.cndatacom.datas;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.text.TextUtils;

public class DBProvider extends ContentProvider
{
	public final int _DIRS = 1; // ƥ
	public final int _ITEM = 2; // 

	private DBHelper _DBHelper = null;
	private SQLiteDatabase _DB = null;
	private UriMatcher _UriMatcher = null;
	public String _TABLE_AUTH = "com.cndatacom.utils.database";
	public String _TABLE_NAME = ""; //ݱ
	public Uri _TABLE_URI = Uri.parse("content://" + _TABLE_AUTH + "/" + _TABLE_NAME);
	
	private String getWhere(Uri uri, String where)
	{
		String id = String.valueOf(ContentUris.parseId(uri));
		if (TextUtils.isEmpty(where))
		{
			where = " [id]= " + id;
		} else {
			where = " [id]= " + id + " AND(" + where + ") ";
		}
		return where;
	}
	
	@Override
	public boolean onCreate() 
	{
		_DBHelper = new DBHelper(getContext(), "", 1);
		
		_DB = _DBHelper.getWritableDatabase();
		
		_UriMatcher = new UriMatcher(UriMatcher.NO_MATCH);		
		_UriMatcher.addURI(_TABLE_AUTH, _TABLE_NAME, _DIRS);
		_UriMatcher.addURI(_TABLE_AUTH, _TABLE_NAME + "/#", _ITEM);
		
		return (null == _DBHelper) ? false : true;
	}

	@Override
	public String getType(Uri uri)
	{
		switch (_UriMatcher.match(uri)) 
		{
			case _DIRS:
				return "vnd.android.cursor.dir/vnd.studio.android." + _TABLE_NAME;
			case _ITEM:
				return "vnd.android.cursor.item/vnd.studio.android." + _TABLE_NAME;
			default:
				throw new IllegalArgumentException("UnknownURI" + uri);
		}
	}

	@Override
	public int delete(Uri uri, String where, String[] args) 
	{
		int count;
		switch (_UriMatcher.match(uri))
		{
			case _DIRS:
				count = _DB.delete(_TABLE_NAME, where, args);
				break;
			case _ITEM:
				where = getWhere(uri, where);
				count = _DB.delete(_TABLE_NAME, where, args);
				break;
			default:
				throw new IllegalArgumentException("UnknownURI" + uri);
		}
		getContext().getContentResolver().notifyChange(uri, null);
		
		return count;
	}

	@Override
	public Uri insert(Uri uri, ContentValues contentvalues)
	{
		long rowId;
		switch (_UriMatcher.match(uri))
		{
			case _DIRS:
				rowId = _DB.insert(_TABLE_NAME, "[id]", contentvalues);
				if (rowId > 0) {
					Uri noteUri = ContentUris.withAppendedId(uri, rowId);
					getContext().getContentResolver().notifyChange(noteUri, null);
					return noteUri;
				} else {
					throw new SQLException("Failedtoinsertrowinto" + uri);
				}
			default:
				throw new IllegalArgumentException("UnknownURI" + uri);
		}
	}

	@Override
	public Cursor query(
			Uri uri, 
			String[] projection, 
			String where,
			String[] args, 
			String sortOrder) {
		switch (_UriMatcher.match(uri)) {
			case _DIRS:
				return _DB.query(_TABLE_NAME, projection, where, args, null, null, sortOrder);
			case _ITEM:
				where = getWhere(uri, where);
				return _DB.query(_TABLE_NAME, projection, where, args, null, null, sortOrder);
			default:
				throw new IllegalArgumentException("UnknownURI" + uri);
		}
	}

	@Override
	public int update(Uri uri, ContentValues values, String where, String[] args)
	{
		int count;
		switch (_UriMatcher.match(uri))
		{		
			case _DIRS:
				count = _DB.update(_TABLE_NAME, values, where, args);
				break;
			case _ITEM:
				where = getWhere(uri, where);
				count = _DB.update(_TABLE_NAME, values, where, args);
				break;
			default:
				throw new IllegalArgumentException("UnknownURI" + uri);
		}
		getContext().getContentResolver().notifyChange(uri, null);
		return count;
	}
}
