package com.cndatacom.network;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.Proxy;
import java.net.Socket;
import java.net.URL;
import java.util.List;
import java.util.Map;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.params.HttpClientParams;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;

import android.content.Context;
import android.text.TextUtils;

import com.cndatacom.crypto.Crypt_Base64;
import com.cndatacom.crypto.Crypto;

/**
 * HTTPؼ
 */
public class Http 
{
	private static int iTimeout = 8000;
	
	public static String postProxy(Context context, String sURL, String sUserAgent, String[] headers, String sBody)
	{
		String results = "";
		try
		{
			StringBuilder sb = new StringBuilder();
			URL url = new URL(sURL);
			HttpURLConnection conn = null;
			Proxy proxy = Utils.getProxy(context);
			if(null != proxy)
			{
				conn = (HttpURLConnection) url.openConnection(proxy);
			} 
			else
			{
				conn = (HttpURLConnection) url.openConnection();
		    }
	
			byte[] entity = sBody.getBytes();
			
			conn.setConnectTimeout(iTimeout);
			conn.setRequestMethod("POST");
			conn.setDoOutput(true);
			conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded;charset=utf-8");
			conn.setRequestProperty("Content-Length", String.valueOf(entity.length));
			if (!TextUtils.isEmpty(sUserAgent))
			{
				conn.setRequestProperty("User-Agent", sUserAgent);
			}
			if (null != headers && headers.length > 0)
			{
				int iHeader = headers.length;
				String sItem;
				String items[] = new String[2];
				for(int i = 0; i < iHeader; i++){
					sItem = headers[i];
					if (null != sItem && !sItem.equals("")){
						items = sItem.split(",");
						if (items.length == 2)
						{
							conn.addRequestProperty(items[0].trim(), items[1].trim());
						}
					}
				}
			}
			OutputStream writer = conn.getOutputStream();
			if (null != writer)
			{
				writer.write(entity);
				writer.flush();
				writer.close();
			}
			InputStream isReader = conn.getInputStream();
			BufferedReader reader = new BufferedReader(new InputStreamReader(isReader));
			String line = null;
	        while (null != (line = reader.readLine()))
	        {
	            sb.append(line);
	        }
	        reader.close();
	        conn.disconnect();
	        results = sb.toString();
		}
		catch(Exception ex)
		{
			results = "error: " + ex.toString();
		}
		
		return results;
	}
	
	public static String post(String sURL, String sBody, String appKey, String selfKey)
	{
		String sResponse = "";
		try
		{
			String sTimestamp = String.valueOf(System.currentTimeMillis());
			String sData = new String(Crypt_Base64.encode(sBody.getBytes("UTF-8")));
			String sign = appKey + sTimestamp + sData;
			String headers[] = new String[3];
			headers[0] = "appKey," + appKey;
			headers[1] = "sign," + Crypto.getHMACSHA1(sign.getBytes(), selfKey.getBytes());
			headers[2] = "timestamp," + sTimestamp;
			ResponseInfo response = postString(sURL, "", headers, sData);
			if (null != response && response.getStatusCode() == 200)
			{
				sResponse = response.getResponseContent();
				if (null != sResponse && !sResponse.equals(""))
				{
					sResponse = new String(Crypt_Base64.decode(sResponse.getBytes("UTF-8")));
				}
			}
		} 
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		return sResponse;
	}
	
	public static ResponseInfo postValues(String sURL, String sUserAgent, String[] headers, List<NameValuePair> nameValuePairs)
	{
		ResponseInfo result = new ResponseInfo();
		HttpClient client = null;
		InputStream is = null;
		ByteArrayOutputStream os = null;
		try
		{
			HttpParams params = new BasicHttpParams();
			HttpConnectionParams.setConnectionTimeout(params, iTimeout);
			HttpConnectionParams.setSoTimeout(params, iTimeout);
			HttpClientParams.setRedirecting(params, true);
			if (!TextUtils.isEmpty(sUserAgent))
			{
				HttpProtocolParams.setUserAgent(params, sUserAgent);
			}
			HttpPost httpPost = new HttpPost(sURL);
			if (null != headers && headers.length > 0)
			{
				int iHeader = headers.length;
				String sItem;
				String items[] = new String[2];
				for(int i = 0; i < iHeader; i++){
					sItem = headers[i];
					if (null != sItem && !sItem.equals("")){
						items = sItem.split(",");
						if (items.length == 2)
						{
							httpPost.addHeader(items[0].trim(), items[1].trim());
						}
					}
				}
			}
			if (null != nameValuePairs && nameValuePairs.size() > 0)
			{
				httpPost.setEntity(new UrlEncodedFormEntity(nameValuePairs, HTTP.UTF_8));
			}
			client = new DefaultHttpClient(params);
			HttpResponse response = client.execute(httpPost);
			if (null != response){
				StatusLine statusCode = response.getStatusLine();
				if (null != statusCode){
					result.setStatusCode(statusCode.getStatusCode());
					HttpEntity entity = response.getEntity();
					if (null != entity)
					{
						is = entity.getContent();
						os = new ByteArrayOutputStream();
						byte[] buffer = new byte[1024];
						int read = 0;
						while ((read = is.read(buffer)) > -1)
						{
							os.write(buffer, 0, read);
						}
						result.setResponseContent(os.toString());
						is.close();
						os.close();
					}
				}
			}
		}
		catch(UnsupportedEncodingException ex)
		{
			ex.printStackTrace();
		}
		catch(ClientProtocolException ex)
		{
			ex.printStackTrace();
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
		catch (Exception ex) 
		{
			ex.printStackTrace();
		} 
		finally 
		{
			if (null != client){
				client.getConnectionManager().shutdown();
			}
		}
		return result;
	}

	public static ResponseInfo postString(String sURL, String sUserAgent, String[] headers, String sBody)
	{
		ResponseInfo result = new ResponseInfo();
		HttpClient client = null;
		InputStream is = null;
		ByteArrayOutputStream os = null;
		try
		{
			HttpParams params = new BasicHttpParams();
			HttpConnectionParams.setConnectionTimeout(params, iTimeout);
			HttpConnectionParams.setSoTimeout(params, iTimeout);
			HttpClientParams.setRedirecting(params, true);
			if (!TextUtils.isEmpty(sUserAgent))
			{
				HttpProtocolParams.setUserAgent(params, sUserAgent);
			}
			HttpPost httpPost = new HttpPost(sURL);
			if (null != headers && headers.length > 0)
			{
				int iHeader = headers.length;
				String sItem;
				String items[] = new String[2];
				for(int i = 0; i < iHeader; i++){
					sItem = headers[i];
					if (null != sItem && !sItem.equals("")){
						items = sItem.split(",");
						if (items.length == 2)
						{
							httpPost.addHeader(items[0].trim(), items[1].trim());
						}
					}
				}
			}
			if (null != sBody && !sBody.equals(""))
			{
				httpPost.setEntity(new StringEntity(sBody, HTTP.UTF_8));
			}
			client = new DefaultHttpClient(params);
			HttpResponse response = client.execute(httpPost);
			if (null != response){
				StatusLine statusCode = response.getStatusLine();
				if (null != statusCode){
					result.setStatusCode(statusCode.getStatusCode());
					HttpEntity entity = response.getEntity();
					if (null != entity)
					{
						is = entity.getContent();
						os = new ByteArrayOutputStream();
						byte[] buffer = new byte[1024];
						int read = 0;
						while ((read = is.read(buffer)) > -1)
						{
							os.write(buffer, 0, read);
						}
						result.setResponseContent(os.toString());
						is.close();
						os.close();
					}
				}
			}
		}
		catch(UnsupportedEncodingException ex)
		{
			ex.printStackTrace();
		}
		catch(ClientProtocolException ex)
		{
			ex.printStackTrace();
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
		catch (Exception ex) 
		{
			ex.printStackTrace();
		} 
		finally 
		{
			if (null != client){
				client.getConnectionManager().shutdown();
			}
		}
		return result;
	}
	
	public static ResponseInfo getURL(String sURL, String sUserAgent, String[] headers)
	{
		ResponseInfo result = new ResponseInfo();
		HttpClient client = null;
		InputStream is = null;
		ByteArrayOutputStream os = null;
		try 
		{
			HttpParams params = new BasicHttpParams();
			HttpConnectionParams.setConnectionTimeout(params, iTimeout);
			HttpConnectionParams.setSoTimeout(params, iTimeout);
			HttpClientParams.setRedirecting(params, true);
			if (null != sUserAgent && !sUserAgent.equals(""))
			{
				HttpProtocolParams.setUserAgent(params, sUserAgent);
			}
			client = new DefaultHttpClient(params);
			HttpGet httpGet = new HttpGet(sURL);
			if (null != headers && headers.length > 0)
			{
				int iHeader = headers.length;
				String sItem;
				String items[] = new String[2];
				for(int i = 0; i < iHeader; i++){
					sItem = headers[i];
					if (null != sItem && !sItem.equals("")){
						items = sItem.split(",");
						if (items.length == 2)
						{
							httpGet.addHeader(items[0].trim(), items[1].trim());
						}
					}
				}
			}
			HttpResponse response = client.execute(httpGet);
			if (null != response){
				StatusLine statusCode = response.getStatusLine();
				if (null != statusCode){
					result.setStatusCode(statusCode.getStatusCode());
					HttpEntity entity = response.getEntity();
					if (null != entity){
						is = entity.getContent();
						os = new ByteArrayOutputStream();
						byte[] buffer = new byte[1024];
						int len = 0;
						while ((len = is.read(buffer)) > -1)
						{
							os.write(buffer, 0, len);
						}
						result.setResponseContent(os.toString());
						os.close();
						is.close();
					}
				}
			}
		} 
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
		catch (Exception ex) 
		{
			ex.printStackTrace();
		} 
		finally 
		{
			if (null != client)
			{
				client.getConnectionManager().shutdown();
			}
		}
		return result;
	}

	/**
	* ֱͨHTTPЭύݵ,ʵύ:
	* <form method="post" action="url" enctype="multipart/form-data">
	* ID: <input type="text" name="id" /><br />
	* <input type="file" name="file1"/><br />
	* <input type="file" name="file2"/>
	* </form>
	* @param sURL ϴ·(עʹlocalhost127.0.0.1·ԣΪָֻģ)
	* @param params  keyΪ,valueΪֵ
	* @param files ϴļ
	*/ 
	public static boolean postForms(String sURL, Map<String, String> params, FileInfo[] files)
	{
		try
		{
			//ݷָ
			final String BOUNDARY = "---------------------------7da2137580612";
			//ݽ־
			final String endline = "--" + BOUNDARY + "--\r\n";  
			int fileDataLength = 0;
			//õļݵܳ
			for(FileInfo file : files)
			{  
				StringBuilder fileExplain = new StringBuilder(); 
				fileExplain.append("--"); 
				fileExplain.append(BOUNDARY);
				fileExplain.append("\r\n");
				fileExplain.append("Content-Disposition:form-data;name=\""+ file.getParameterName()+"\";filename=\""+ file.getFilname() + "\"\r\n"); 
				fileExplain.append("Content-Type: "+ file.getContentType()+"\r\n\r\n"); 
				fileExplain.append("\r\n"); 
				fileDataLength += fileExplain.length(); 
				if(null != file.getInStream()){ 
					fileDataLength += file.getFile().length(); 
				}else{ 
					fileDataLength += file.getData().length; 
				} 
			} 
			StringBuilder textEntity = new StringBuilder();
			//ıͲʵ
			for (Map.Entry<String, String> entry : params.entrySet())
			{  
				textEntity.append("--"); 
				textEntity.append(BOUNDARY);
				textEntity.append("\r\n"); 
				textEntity.append("Content-Disposition:form-data;name=\""+ entry.getKey() + "\"\r\n\r\n"); 
				textEntity.append(entry.getValue());
				textEntity.append("\r\n"); 
			} 
			int dataLength = textEntity.toString().getBytes().length + fileDataLength +  endline.getBytes().length; 
			URL url = new URL(sURL); 
			int port = url.getPort()==-1 ? 80 : url.getPort(); 
			Socket socket = new Socket(InetAddress.getByName(url.getHost()), port);         
			OutputStream outStream = socket.getOutputStream(); 
			String requestmethod = "POST "+ url.getPath()+" HTTP/1.1\r\n";
			outStream.write(requestmethod.getBytes()); 
			String accept = "Accept: image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, application/xaml+xml, application/vnd.ms-xpsdocument, application/x-ms-xbap, application/x-ms-application, application/vnd.ms-excel, application/vnd.ms-powerpoint, application/msword, */*\r\n"; 
			outStream.write(accept.getBytes()); 
			String language = "Accept-Language: zh-CN\r\n"; 
			outStream.write(language.getBytes()); 
			String contenttype = "Content-Type:multipart/form-data;boundary="+ BOUNDARY+ "\r\n"; 
			outStream.write(contenttype.getBytes()); 
			String contentlength = "Content-Length: "+ dataLength + "\r\n"; 
			outStream.write(contentlength.getBytes()); 
			String alive = "Connection: Keep-Alive\r\n"; 
			outStream.write(alive.getBytes()); 
			String host = "Host: "+ url.getHost() +":"+ port +"\r\n"; 
			outStream.write(host.getBytes()); 
			//дHTTPͷHTTPЭдһس  
			outStream.write("\r\n".getBytes()); 
			//ı͵ʵݷͳ  
			outStream.write(textEntity.toString().getBytes());
			//ļ͵ʵݷͳ  
			for(FileInfo file : files)
			{ 
				StringBuilder fileEntity = new StringBuilder(); 
				fileEntity.append("--"); 
				fileEntity.append(BOUNDARY); 
				fileEntity.append("\r\n"); 
				fileEntity.append("Content-Disposition: form-data;name=\""+ file.getParameterName()+"\";filename=\""+ file.getFilname() + "\"\r\n"); 
				fileEntity.append("Content-Type: "+ file.getContentType()+"\r\n\r\n"); 
				outStream.write(fileEntity.toString().getBytes()); 
				if(file.getInStream()!=null){ 
					byte[] buffer = new byte[1024]; 
					int len = 0; 
					while((len = file.getInStream().read(buffer, 0, 1024))!=-1)
					{ 
						outStream.write(buffer, 0, len); 
					} 
					file.getInStream().close(); 
				}else{ 
					outStream.write(file.getData(), 0, file.getData().length); 
				} 
				outStream.write("\r\n".getBytes()); 
			} 
			//淢ݽ־ʾѾ  
			outStream.write(endline.getBytes());
			BufferedReader reader = new BufferedReader(new InputStreamReader(socket.getInputStream())); 
			//ȡwebصݣжǷΪ200200ʧ
			if(reader.readLine().indexOf("200")==-1)
			{  
				return false; 
			} 
			outStream.flush();
			outStream.close(); 
			reader.close(); 
			socket.close();
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		return true; 
	} 
}
