package com.cndatacom.trees;

import java.util.List;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class TreesAdapter<T> extends ArrayAdapter<TreesModel>
{
	public TreesAdapter(Context context, int textViewResourceId, List<TreesModel> lists, int icon1, int icon2)
	{
		super(context, textViewResourceId, lists);
		
		inflater = LayoutInflater.from(context);
		nodes = lists;
		iconOpen  = BitmapFactory.decodeResource(context.getResources(), icon1);
		iconClose = BitmapFactory.decodeResource(context.getResources(), icon2);
	}

	private LayoutInflater inflater;
	private List<TreesModel> nodes;
	private Bitmap iconOpen;
	private Bitmap iconClose;

	public int getCount()
	{
		return nodes.size();
	}

	public TreesModel getItem(int position)
	{
		return nodes.get(position);
	}

	public long getItemId(int position)
	{
		return position;
	}

	public View getView(int position, View convertView, ViewGroup parent, int layout, int text, int icon) 
	{
		ViewHolder holder = new ViewHolder();
		convertView = inflater.inflate(layout, null);
		holder.text = (TextView)  convertView.findViewById(text);
		holder.icon = (ImageView) convertView.findViewById(icon);
		convertView.setTag(holder);

		TreesModel node = nodes.get(position);
		holder.icon.setPadding(
			25 * node.Level,
			holder.icon.getPaddingTop(),
			0,
			holder.icon.getPaddingBottom()
		);
		holder.text.setText(node.Title);
		if (node.HasChild)
		{
			if (node.IsExpanded)
			{
				holder.icon.setImageBitmap(iconClose);
			} 
			else
			{
				holder.icon.setImageBitmap(iconOpen);
			} 
		}
		else
		{
			holder.icon.setImageBitmap(iconOpen);
			holder.icon.setVisibility(View.INVISIBLE);
		}
		return convertView;
	}

	public class ViewHolder 
	{
		TextView text;
		ImageView icon;
	}
}