package com.cndatacom.wifi;

import java.util.List;

import com.cndatacom.utils.Strings;
import com.cndatacom.utils.Tools;

import android.app.Service;
import android.content.Context;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.net.wifi.WifiManager.WifiLock;
import android.os.SystemClock;
import android.widget.ArrayAdapter;

public class WifiUtils
{
	private Context _Context;
	private WifiManager _Wifi;
	private WifiLock _Lock;
	/**
	 * 캯
	 * @param context
	 */
	public WifiUtils(Context context)
	{
		this._Context = context;
		this._Wifi = (WifiManager) this._Context.getSystemService(Service.WIFI_SERVICE);
	}
	/**
	 * 
	 * @return
	 */
	public List<ScanResult> getScanResults()
	{
		List<ScanResult> list = null;
		if (getEnableStatus())
		{
			list = _Wifi.getScanResults();
		}
		return list;
	}
	/**
	 * 
	 * @return
	 */
	public ArrayAdapter<String> getSSIDList()
    {
    	return getSSIDList(getScanResults());
    }
	/**
	 * 
	 * @param results
	 * @return
	 */
	public ArrayAdapter<String> getSSIDList(List<ScanResult> results)
    {
		ArrayAdapter<String> adpList = null;
		if (null != results)
		{
			int counts = results.size();
			String[] SSID = new String[counts];
			for(int i = 0; i < counts; i++){
				SSID[i] = results.get(i).SSID;
			}
	    	adpList = getSSIDList(SSID);
		}
    	return adpList;
    }
	/**
	 * 
	 * @param SSID
	 * @return
	 */
	public ArrayAdapter<String> getSSIDList(String[] SSID)
    {
		ArrayAdapter<String> adpList = null;
		if (null != SSID)
		{
			adpList = new ArrayAdapter<String>(_Context, android.R.layout.simple_list_item_1, SSID);
		}
    	return adpList;
    }
	/**
	 * ȡWIFI״̬
	 * @return
	 */
	public boolean getEnableStatus()
	{
		return _Wifi.isWifiEnabled();
	}
	/**
	 * WIFI״̬
	 * @param status
	 */
	public void setStatus(boolean status)
	{
		_Wifi.setWifiEnabled(status);
	}
	/**
	 * WIFI
	 * @return
	 */
	public boolean enable(){
		boolean results = _Wifi.isWifiEnabled();
		if (!results){
			_Wifi.setWifiEnabled(true);
		}
		int iTimeout = 5;
		while(!results){
			iTimeout -= 1;
			results = _Wifi.isWifiEnabled();
			SystemClock.sleep(1000);
			if (iTimeout <= 0){ break; }
		}
		return results;
	}	
	/**
     * 
     * @param SSID
     * @param SKEY
     * @return
     */
    public int connect(String SSID)
    {
    	int results = 1;
    	if (!Strings.isEmpty(SSID)){
        	int networkID = 0;
	    	WifiConfiguration config = getConfig(SSID);
	        if (null == config){
		    	config = new WifiConfiguration();  
		        config.allowedAuthAlgorithms.clear();
		        config.allowedGroupCiphers.clear();
		        config.allowedKeyManagement.clear();
		        config.allowedPairwiseCiphers.clear();
		        config.allowedProtocols.clear();
		        config.SSID = "\"" + SSID + "\"";
		        config.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);		        
		        config.status = WifiConfiguration.Status.ENABLED;
		    	networkID = _Wifi.addNetwork(config);
				System.out.println("add NetworkId: " + networkID);
	        } else {
	        	networkID = config.networkId;
	        }

	    	if (networkID > 0){
	    		boolean enableNetworkStatus = _Wifi.enableNetwork(networkID, true);
				if (enableNetworkStatus) {
					_Lock = _Wifi.createWifiLock(1, SSID);
					_Lock.acquire();
					results = 0;
				}
				System.out.println("enableNetwork: " + enableNetworkStatus);
	    	}
    	}
    	return results;
    }
	/**
	 * ȡSSIDǷѾб
	 * @param SSID
	 * @return
	 */
	public WifiConfiguration getConfig(String SSID)  
    {
        List<WifiConfiguration> existingConfigs = _Wifi.getConfiguredNetworks();  
        for (WifiConfiguration existingConfig : existingConfigs)   
        {
        	if (existingConfig.SSID.equals("\""+SSID+"\""))
            {
        		return existingConfig;
            }  
        }  
        return null;
    }
    /**
     * 
     * @param tagLock
     * @return
     */
    public boolean setWifiLock(String tagLock)
    {
    	boolean results = false;    	
    	WifiInfo current = Tools.getConnectWifiInfo(_Context);
    	if (null != current){
    		results = true;
			if (results) {
				_Lock = _Wifi.createWifiLock(1, tagLock);
				_Lock.acquire();
			}
    	}
    	return results;
    }
    /**
     * Ͽ
     */
    public void disconnect(){
    	WifiInfo current = Tools.getConnectWifiInfo(_Context);
    	if (null != current){
	    	if (_Lock.isHeld()){
	    		_Lock.release();
	    	}
	    	_Wifi.disableNetwork(current.getNetworkId());
    	}
    }
}
